import java.util.Arrays;
import java.util.Random;

public class RadixSort {
	final static int numberOfElements = 10;
	final static int base = 10;

	public static void radixSortForIntegersByPlaceWithBase(int [] original, int [] sorted, int place) {

		int [] bucket = new int[base];
		Arrays.fill(bucket, 0);

		for (int i = 0; i < original.length; i++) {
			bucket[(original[i] / place) % base]++;
		}
		for (int i = 1; i < bucket.length; i++) {
			bucket[i] += bucket[i-1];
		}
		for (int i = original.length-1 ; i >= 0; i--) {
			sorted[--bucket[(original[i] / place) % base]] = original[i];
		}
	}
	public static void radixSortForIntegersByPlace(int [] original, int [] sorted, int maxNumber) {
		int multiplier = 1;

		while (multiplier <= maxNumber) {
			radixSortForIntegersByPlaceWithBase(original, sorted, multiplier);
			for (int i = 0; i < sorted.length; i++) {
				original[i] = sorted[i];
			}
			multiplier *= base;
		}
	}
	public static float [] getFloatArray() {
		final int maxValue = 2000000;
		float [] array = new float[numberOfElements];

		Random rand = new Random();
		for (int i = 0; i < numberOfElements; i++) {
			int decimalPoints = rand.nextInt(3)+1;
			boolean sign = rand.nextBoolean();
			int nextInt = rand.nextInt(maxValue);
			array[i] = (nextInt / (float)Math.pow(base, decimalPoints)) * (sign ? 1: -1);
		}

		return array;
	}
	public static int [] getIntArray() {
		final int maxValue = 200 * base;
		int [] intArray = new int[numberOfElements];

		Random rand = new Random();
		for (int i = 0; i < numberOfElements; i++) {
			boolean sign = rand.nextBoolean();
			int nextInt = rand.nextInt(maxValue);
			intArray[i] = nextInt * (sign ? 1: -1);
		}

		return intArray;
	}
	public static String getStringOfLength(int maxStrLength, String type) {
		Random rand = new Random();
		String str = "";
		int minValue = 32;
		int maxValue = 127;
		int strSize = rand.nextInt(maxStrLength);

		while (str.length() < strSize) {
			int ascii = rand.nextInt(255)+1;

			if (type.contentEquals("alphanumeric")) {
				if (ascii == ' ' ||  (ascii >= 'a' && ascii <= 'z') ||
					(ascii >= 'A' && ascii <= 'Z') ||
					(ascii >= '0' && ascii <= '9')
				)
					str += (char)(ascii);
			} else if (ascii >= minValue && ascii < maxValue)
				str += (char)(ascii);
		}
		return str;
	}
	public static String getAlphanumericStringOfLength(int maxStrLength) {
		return getStringOfLength(maxStrLength, "alphanumeric");
	}
	public static String [] getStringArray() {
		final int maxStringSize = 15;
		String [] strArray = new String[numberOfElements];

		for (int i = 0; i < numberOfElements; i++) {
			strArray[i] = getAlphanumericStringOfLength(maxStringSize);
		}

		return strArray;
	}
	public static void printBanner(String label) {
		int bannerLength = 110;
		StringBuffer stars = new StringBuffer();
		StringBuffer dashes = new StringBuffer();
		String descformat = "%" + String.format("%d", (bannerLength-label.length())/2 + label.length()) + "s\n";
		for (int i = 0; i < bannerLength; i++) stars.append('*');
		for (int i = 0; i < bannerLength; i++) dashes.append('-');

		System.out.println(stars);
		System.out.printf(descformat, label);
		System.out.println(dashes);
	}
	public static void printFloatArray(float [] array, String label) {
		StringBuffer sb = new StringBuffer();
		for (float num: array) {
			if (sb.length() > 0)
				sb.append(", ");
			sb.append("" + num);
		}
		printBanner(label);
		System.out.printf("{ %s }\n", sb);
	}
	public static void printIntArray(int [] intArray, String label) {
		StringBuffer sb = new StringBuffer();
		for (int i: intArray) {
			if (sb.length() > 0)
				sb.append(", ");
			sb.append("" + i);
		}
		printBanner(label);
		System.out.printf("{ %s }\n", sb);
	}
	public static void printStringArray(String [] strArray, String label) {
		StringBuffer sb = new StringBuffer();
		for (String str: strArray) {
			if (sb.length() > 0)
				sb.append(", ");
			sb.append("\n\"" + str.trim() + "\"");
		}
		printBanner(label);
		System.out.printf("[ %s \n]", sb);
	}
	public static int getMinInteger(int [] intArray) {
		int minInteger = intArray[0];

		for (int i: intArray) {
			if (i < minInteger)
				minInteger = i;
		}
		return minInteger;
	}
	public static void convertIntegers(int [] intArray, int minInteger) {
		for (int i = 0; i < intArray.length; i++) {
			intArray[i] += minInteger;
		}
	}

	public static int getMaxNumberInPositiveArray(int [] intArray) {
		int maxNumber = intArray[0];

		for (int num: intArray) {
			if (maxNumber < num)
				maxNumber = num;
		}
		return maxNumber;
	}

	public static void radixSortForStringByIndexCount(String [] original, String [] sorted, int index) {
		int [] buckets = new int[256];
		Arrays.fill(buckets,  0);

		for (int i = 0; i < original.length; i++) {
			buckets[(int)original[i].charAt(index)]++;
		}
		for (int i = 1; i < buckets.length; i++) {
			buckets[i] += buckets[i-1];
		}
		for (int i = original.length-1 ; i >= 0; i--) {
			sorted[--buckets[(int)original[i].charAt(index)]] = original[i];
		}
	}
	public static void radixSortForStringByIndex(String [] original, String [] sorted, int maxStringLength) {
		for (int index = maxStringLength-1; index >= 0; index--) {
			radixSortForStringByIndexCount(original, sorted, index);
			for (int i = 0; i < original.length; i++) {
				original[i] = sorted[i];
			}
		}
	}

	public static int getMaxStringLength(String [] original) {
		int maxStringLength = 0;

		for (String str: original) {
			if (str.length() > maxStringLength)
				maxStringLength = str.length();
		}
		return maxStringLength;
	}

	public static void convertStrings(String [] original, int maxStringLength) {
		for (int i = 0; i < original.length; i++) {
			while (original[i].length() < maxStringLength) {
				original[i] += " ";
			}
		}
	}
	public static String [] radixSortForStrings(String [] original) {
		String [] sorted = new String[original.length];
		int maxStringLength = getMaxStringLength(original);
		convertStrings(original, maxStringLength);
		radixSortForStringByIndex(original, sorted, maxStringLength);

		return sorted;
	}

	public static int getMaxDecimalPlaces(float [] original) {
		int maxDecimalPlaces = 0;
		for (int i = 0; i < original.length; i++) {
			String floatAsString = Float.toString(original[i]);
			int index = floatAsString.indexOf('.');
			if (index >= 0) {
				if (floatAsString.substring(index+1).length() > maxDecimalPlaces)
					maxDecimalPlaces = floatAsString.substring(index+1).length();
			}
			System.out.printf("original %f str  %s index %d\n", original[i], floatAsString, index);

		}
		return maxDecimalPlaces;
	}
	public static float [] radixSortForFloats(float [] original) {
		int [] originalAsInts = new int[original.length];

		for (int i = 0; i < original.length; i++) {
			originalAsInts[i] = (int)(original[i]*1000);
		}
		int [] sortedInts = radixSortForIntegers(originalAsInts);
		float [] sortedFloats = new float[sortedInts.length];

		for (int i = 0; i < original.length; i++) {
			sortedFloats[i] = (float)(sortedInts[i] / 1000.0);

		}
		return sortedFloats;
	}

	public static int [] radixSortForIntegers(int [] original) {
		int [] sorted = new int[original.length];

		int minInteger = getMinInteger(original);


		if (minInteger < 0) {
			convertIntegers(original, minInteger*-1);
		}
		int maxNumber = getMaxNumberInPositiveArray(original);
		radixSortForIntegersByPlace(original, sorted, maxNumber);
		if (minInteger < 0) {
			convertIntegers(sorted, minInteger);
		}
		return sorted;
	}

	public static void testIntegers() {
		int [] original = getIntArray();

		System.out.println();
		printIntArray(original, "Original Integer Array");
		int [] sorted = radixSortForIntegers(original);
		printIntArray(sorted, "Sorted Integer Array");
	}

	public static void testFloats() {
		float [] original = getFloatArray();

		System.out.println();
		printFloatArray(original, "Original Float Array");
		float [] sorted = radixSortForFloats(original);
		printFloatArray(sorted, "Sorted Float Array");
	}
	public static void testStrings() {
		String [] original = getStringArray();

		System.out.println();
		printStringArray(original, "Original String Array");
		String [] sorted = radixSortForStrings(original);
		printStringArray(sorted, "Sorted String Array");
	}

	public static void main(String [] args) {
		testIntegers();
		testStrings();
		testFloats();
	}
}
